import { Point } from '../rga/Point';
import { OverlayPoint } from './OverlayPoint';
import { MarkerOverlayPoint } from './MarkerOverlayPoint';
import { ITimestampStruct } from '../../../json-crdt-patch/clock';
import { Range } from '../rga/Range';
import type { Chunk } from '../../../json-crdt/nodes/rga';
import type { Peritext } from '../Peritext';
import type { Stateful } from '../types';
import type { Printable } from 'tree-dump/lib/types';
import type { Slice } from '../slice/types';
/**
 * Overlay is a tree structure that represents all the intersections of slices
 * in the text. It is used to quickly find all the slices that overlap a
 * given point in the text. The overlay is a read-only structure, its state
 * is changed only by calling the `refresh` method, which updates the overlay
 * based on the current state of the text and slices.
 */
export declare class Overlay<T = string> implements Printable, Stateful {
    protected readonly txt: Peritext<T>;
    root: OverlayPoint<T> | undefined;
    readonly start: OverlayPoint<T>;
    constructor(txt: Peritext<T>);
    private point;
    private mPoint;
    first(): OverlayPoint<T> | undefined;
    last(): OverlayPoint<T> | undefined;
    iterator(): () => OverlayPoint<T> | undefined;
    entries(): IterableIterator<OverlayPoint<T>>;
    [Symbol.iterator](): IterableIterator<OverlayPoint<T>>;
    markerIterator(): () => MarkerOverlayPoint | undefined;
    markers(): IterableIterator<OverlayPoint<T>>;
    /**
     * Retrieve overlay point or the previous one, measured in spacial dimension.
     */
    getOrNextLower(point: Point<T>): OverlayPoint<T> | undefined;
    /**
     * Retrieve overlay point or the next one, measured in spacial dimension.
     */
    getOrNextHigher(point: Point<T>): OverlayPoint<T> | undefined;
    find(predicate: (point: OverlayPoint<T>) => boolean): OverlayPoint<T> | undefined;
    chunkSlices0(chunk: Chunk<T> | undefined, p1: Point<T>, p2: Point<T>, callback: (chunk: Chunk<T>, off: number, len: number) => void): Chunk<T> | undefined;
    points0(start: undefined | OverlayPoint<T>, end: undefined | ((next: OverlayPoint<T>) => boolean), callback: (point: OverlayPoint<T>) => void): void;
    points1(start: undefined | OverlayPoint<T>, end: undefined | ((next: OverlayPoint<T>) => boolean), callback: (p1: OverlayPoint<T>, p2: OverlayPoint<T>) => void): void;
    findContained(range: Range<T>): Set<Slice<T>>;
    findOverlapping(range: Range<T>): Set<Slice<T>>;
    leadingTextHash: number;
    protected computeSplitTextHashes(): void;
    isBlockSplit(id: ITimestampStruct): boolean;
    hash: number;
    refresh(slicesOnly?: boolean): number;
    readonly slices: Map<Slice<T>, [start: OverlayPoint<T>, end: OverlayPoint<T>]>;
    private refreshSlices;
    private insSlice;
    private insMarker;
    private delSlice;
    /**
     * Retrieve an existing {@link OverlayPoint} or create a new one, inserted
     * in the tree, sorted by spatial dimension.
     */
    private upsertPoint;
    /**
     * Inserts a point into the tree, sorted by spatial dimension.
     * @param point Point to insert.
     * @returns Returns the existing point if it was already in the tree.
     */
    private insPoint;
    private delPoint;
    toString(tab?: string): string;
}
